.. image:: https://badge.fury.io/py/dwave-hybrid.svg
    :target: https://badge.fury.io/py/dwave-hybrid
    :alt: Last version on PyPI

.. image:: https://circleci.com/gh/dwavesystems/dwave-hybrid.svg?style=svg
    :target: https://circleci.com/gh/dwavesystems/dwave-hybrid
    :alt: Linux/Mac build status

.. image:: https://ci.appveyor.com/api/projects/status/porqyytww2elwjv8/branch/master?svg=true
    :target: https://ci.appveyor.com/project/dwave-adtt/dwave-hybrid/branch/master
    :alt: Windows build status

.. image:: https://readthedocs.com/projects/d-wave-systems-dwave-hybrid/badge/?version=latest
    :target: https://docs.ocean.dwavesys.com/projects/hybrid/en/latest/
    :alt: Documentation Status


=============
D-Wave Hybrid
=============

.. index-start-marker

A general, minimal Python framework for building hybrid asynchronous decomposition
samplers for quadratic unconstrained binary optimization (QUBO) problems.

*dwave-hybrid* facilitates three aspects of solution development:

*   Hybrid approaches to combining quantum and classical compute resources
*   Evaluating a portfolio of algorithmic components and problem-decomposition strategies
*   Experimenting with workflow structures and parameters to obtain the best application results

The framework enables rapid development and insight into expected performance
of productized versions of its experimental prototypes.

Your optimized algorithmic components and other contributions to this project are welcome!

.. index-end-marker


Installation or Building
========================

.. installation-start-marker

Install from a package on PyPI::

    pip install dwave-hybrid

or from source::

    git clone https://github.com/dwavesystems/dwave-hybrid.git
    cd dwave-hybrid
    pip install -r requirements.txt
    python setup.py install

.. installation-end-marker


Example
=======

.. example-start-marker

.. code-block:: python

    import dimod
    from hybrid.samplers import (
        QPUSubproblemAutoEmbeddingSampler, InterruptableTabuSampler)
    from hybrid.decomposers import EnergyImpactDecomposer
    from hybrid.composers import SplatComposer
    from hybrid.core import State
    from hybrid.flow import RacingBranches, ArgMin, Loop
    from hybrid.utils import min_sample

    # Construct a problem
    bqm = dimod.BinaryQuadraticModel({}, {'ab': 1, 'bc': -1, 'ca': 1}, 0, dimod.SPIN)

    # Define the solver
    iteration = RacingBranches(
        InterruptableTabuSampler(),
        EnergyImpactDecomposer(size=2)
        | QPUSubproblemAutoEmbeddingSampler()
        | SplatComposer()
    ) | ArgMin()
    main = Loop(iteration, max_iter=10, convergence=3)

    # Solve the problem
    init_state = State.from_sample(min_sample(bqm), bqm)
    solution = main.run(init_state).result()

    # Print results
    print("Solution: sample={s.samples.first}".format(s=solution))


.. example-end-marker


License
=======

Released under the Apache License 2.0. See `<LICENSE>`_ file.
